@@ -1,11 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import Model, fields
 from trytond.pool import Pool
 
 
-class Move(ModelSQL, ModelView):
+class Move(Model):
     _name = 'stock.move'
 
     account_move = fields.Many2One('account.move', 'Account Move',
@@ -17,12 +17,14 @@ class Move(ModelSQL, ModelView):
         '''
         uom_obj = Pool().get('product.uom')
         currency_obj = Pool().get('currency.currency')
-        assert type_.startswith('in_') or type_.startswith('out_'), 'wrong type'
+        assert type_.startswith('in_') or type_.startswith('out_'), \
+            'wrong type'
 
         move_line = {
             'name': move.rec_name,
         }
-        if type_.endswith('supplier'):
+        if (type_.endswith('supplier')
+                and move.product.cost_price_method != 'fixed'):
             unit_price = move.unit_price
         else:
             unit_price = uom_obj.compute_price(move.product.default_uom,
@@ -52,6 +54,8 @@ class Move(ModelSQL, ModelView):
             'name': move.rec_name,
             'account': move.product.account_stock_used.id,
         }
+        if not amount:
+            return
         if amount >= Decimal('0.0'):
             move_line['debit'] = Decimal('0.0')
             move_line['credit'] = amount
@@ -60,25 +64,18 @@ class Move(ModelSQL, ModelView):
             move_line['credit'] = Decimal('0.0')
         return move_line
 
-    def _get_account_stock_move(self, move, move_lines, type_):
+    def _get_account_stock_move(self, move, move_lines):
         '''
         Return account move value for stock move
         '''
         date_obj = Pool().get('ir.date')
         period_obj = Pool().get('account.period')
-        assert type_.startswith('in_') or type_.startswith('out_'), 'wrong type'
+        account_configuration_obj = Pool().get('account.configuration')
 
         date = move.effective_date or date_obj.today()
         period_id = period_obj.find(move.company.id, date=date)
-
-        if type_.startswith('in_'):
-            journal_type = type_[3:]
-        else:
-            journal_type = type_[4:]
-
-        journal_id = getattr(move.product,
-                'account_journal_stock_%s_used' % journal_type).id
-
+        account_configuration = account_configuration_obj.browse(1)
+        journal_id = account_configuration.stock_journal.id
         return {
             'journal': journal_id,
             'period': period_id,
@@ -103,32 +100,58 @@ class Move(ModelSQL, ModelView):
             return 'out_lost_found'
         elif type_ == ('lost_found', 'storage'):
             return 'in_lost_found'
+        elif type_ == ('supplier', 'customer'):
+            return 'supplier_customer'
+        elif type_ == ('customer', 'supplier'):
+            return 'customer_supplier'
 
     def _create_account_stock_move(self, move):
         '''
         Create account move for stock move
         '''
         account_move_obj = Pool().get('account.move')
+        if move.product.type != 'goods':
+            return
         type_ = self._get_account_stock_move_type(move)
         if not type_:
             return
         assert not move.account_move, 'account move field not empty'
-        account_move_lines = self._get_account_stock_move_lines(move, type_)
+        if type_ == 'supplier_customer':
+            account_move_lines = self._get_account_stock_move_lines(move,
+                'in_supplier')
+            account_move_lines.extend(self._get_account_stock_move_lines(move,
+                    'out_customer'))
+        elif type_ == 'customer_supplier':
+            account_move_lines = self._get_account_stock_move_lines(move,
+                'in_customer')
+            account_move_lines.extend(self._get_account_stock_move_lines(move,
+                    'out_supplier'))
+        else:
+            account_move_lines = self._get_account_stock_move_lines(move,
+                type_)
 
         amount = Decimal('0.0')
         for line in account_move_lines:
             amount += line['debit'] - line['credit']
-        account_move_lines.append(
-                self._get_account_stock_move_line(move, amount))
+        move_line = self._get_account_stock_move_line(move, amount)
+        if move_line:
+            account_move_lines.append(move_line)
 
         account_move_id = account_move_obj.create(
-                self._get_account_stock_move(move, account_move_lines, type_))
+                self._get_account_stock_move(move, account_move_lines))
         self.write(move.id, {
             'account_move': account_move_id,
             })
         account_move_obj.post(account_move_id)
         return account_move_id
 
+    def copy(self, ids, default=None):
+        if default is None:
+            default = {}
+        default = default.copy()
+        default.setdefault('account_move', None)
+        return super(Move, self).copy(ids, default=default)
+
     def create(self, vals):
         new_id = super(Move, self).create(vals)
         if vals.get('state') == 'done':
